package ibcontroller;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.Socket;
import java.net.SocketException;

class CommandChannel {

    private static final String _Prompt = Settings.getString("CommandPrompt", "");
    private static final boolean _SuppressInfo = Settings.getBoolean("SuppressInfoMessages", true);

    private Socket mSocket;
    private BufferedReader mInstream = null;
    private BufferedWriter mOutstream = null;

    CommandChannel(Socket socket) {

        mSocket = socket;
        if (! setupStreams()) return;

        writeInfo("IBController Server");
        writePrompt();

    }

    void close() {
        try {
            mSocket.shutdownInput();
            mSocket.shutdownOutput();

            mInstream.close();
            mInstream = null;

            mOutstream.close();
            mOutstream = null;

            mSocket.close();
            mSocket = null;
        } catch (SocketException e) {
            // the socket was reset by the client - ignore
        } catch (IOException e) {
            // ignore
        }
    }

    String getCommand() {
        String cmd = null;

        if (mInstream == null) return null;

        try {
            cmd = mInstream.readLine();
            while (cmd != null && cmd.trim().isEmpty()) {
                writePrompt();
                cmd = mInstream.readLine();
            }

            if (cmd != null) Utils.logToConsole("IBControllerServer received command: " + cmd);
        } catch (SocketException e) {
            // the socket was reset by the client
            close();
        } catch (IOException e) {
            close();
        }
        return cmd;
    }

    void writeAck(String info) {
        replyLine("OK " + info);
    }

    void writeInfo(String info) {
        if (! _SuppressInfo) replyLine("INFO " + info);
    }

    void writeNack(String info) {
        replyLine("ERROR " + info);
    }

    void writePrompt() {
        if (! _Prompt.isEmpty()) reply(_Prompt);
    }

    private void reply(String message) {
        if (mOutstream == null) return;
        try {
            mOutstream.write(message);
            mOutstream.flush();
        } catch (SocketException e) {
            // the socket was reset by the client
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void replyLine(String message) {
        reply(message + "\r\n");
    }

    private boolean setupStreams() {
        try {
            mInstream = new BufferedReader(new InputStreamReader(mSocket.getInputStream()));
            mOutstream = new BufferedWriter(new OutputStreamWriter(mSocket.getOutputStream()));
            return true;
        } catch (IOException e) {
            // this is most likely a result of the user closing the command connection
            return false;
        }
    }

}
