package ibcontroller;

import java.util.concurrent.ExecutionException;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

class CommandDispatcher
        implements Runnable {

    private CommandChannel mChannel;
    private boolean mGateway;

    CommandDispatcher(CommandChannel channel, boolean gateway) {
        this.mChannel = channel;
        this.mGateway = gateway;
    }

    @Override public void run() {

        String cmd;

        cmd = mChannel.getCommand();
        while (cmd != null) {
            if (cmd.equalsIgnoreCase("EXIT")) {
                mChannel.writeAck("Goodbye");
                break;
            } else if (cmd.equalsIgnoreCase("STOP")) {
                handleStopCommand();
            } else if (cmd.equalsIgnoreCase("ENABLEAPI")) {
                handleEnableAPICommand();
            } else {
                mChannel.writeNack("Command invalid");
                System.err.println(
                        "IBControllerServer: invalid command received: "
                        + cmd);
            }
            mChannel.writePrompt();
            cmd = mChannel.getCommand();
        }
        mChannel.close();
    }

    private void handleEnableAPICommand() {
        if (mGateway) {
            mChannel.writeNack("ENABLEAPI is not valid for the IB Gateway");
            return;
        }

        Future<?> f = (Executors.newSingleThreadExecutor()).submit(new ConfigureApiTask(mChannel));

        // wait for the task to complete
        try{
            f.get();
        } catch (InterruptedException ie) {
        } catch (ExecutionException ee) {
            ee.printStackTrace();
        }
   }

    private void handleStopCommand() {
        (new GuiExecutor()).execute(new StopTask(mGateway, mChannel));
    }

}
