
package ibcontroller;

import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;

class StopTask
        implements Runnable {

    private static SwitchLock _Running = new SwitchLock();

    private final boolean mGateway;
    private final CommandChannel mChannel;

    public StopTask(boolean gateway, final CommandChannel channel) {
        this.mGateway = gateway;
        mChannel = channel;
    }

    public void run() {
        if (! _Running.set()) {
            if (! (mChannel == null)) mChannel.writeNack("STOP already in progress");
            return;
        }

        try {
            if (! (mChannel == null)) mChannel.writeInfo("Closing IBController");
            stop((mGateway) ? "Close" : "Exit");
        } catch (Exception ex) {
            if (! (mChannel == null)) mChannel.writeNack(ex.getMessage());
        } finally {
            _Running.clear();
        }
    }

    private void stop(String stopCommand) {
        JFrame jf = TwsListener.getMainWindow();
        if (jf == null) {
            Utils.logToConsole("main window not yet found");
            return;
        }

        JMenuBar jmb = Utils.findMenuBar(jf);
        if (jmb == null) {
            System.err.println("IBController: Could not find JMenuBar inside main window.");
            return;
        }

        JMenuItem jmi = Utils.findMenuItem(jmb, "File");
        if (jmi == null) {
            System.err.println("IBController: Could not find File menu inside menubar.");
            return;
        }

        jmi.doClick();

        jmi = Utils.findMenuItem(jmi, stopCommand);
        if (jmi == null) {
            System.err.println("IBController: Could not find " + stopCommand + " menu inside File menu.");
            return;
        }

        jmi.doClick();
    }

}
