
package ibcontroller;

import java.awt.AWTEvent;
import java.awt.Window;
import java.awt.event.AWTEventListener;
import java.awt.event.WindowEvent;
import java.util.List;
import javax.swing.JDialog;
import javax.swing.JFrame;

class TwsListener
        implements AWTEventListener {

    private static final TwsListener _OnlyInstance = new TwsListener();
    private static volatile String _UserName;
    private static volatile String _Password;

    private static List<WindowHandler> _WindowHandlers;

    private static volatile JFrame _LoginFrame = null;
    private static volatile JFrame _MainWindow = null;

    private TwsListener() {}

    static TwsListener getInstance() {return _OnlyInstance; }

    static void initialise(String userName, String password, List<WindowHandler> windowHandlers) {
        _UserName = userName;
        _Password = password;
        _WindowHandlers = windowHandlers;
    }

    public void eventDispatched(AWTEvent event) {
        int eventID = event.getID();
        if (eventID != WindowEvent.WINDOW_OPENED &&
                eventID != WindowEvent.WINDOW_ACTIVATED) return;

        Window window =((WindowEvent) event).getWindow();
        logWindow(window, eventID);

        for (WindowHandler wh : _WindowHandlers) {
            if (wh.recogniseWindow(window)) {
                wh.handleWindow(window, eventID);
                break;
            }
        }

    }

    static JFrame getLoginFrame() {
        return _LoginFrame;
    }

    static JFrame getMainWindow() {
        return _MainWindow;
    }

    static String getPassword() {
        return _Password;
    }

    static String getUserName() {
        return _UserName;
    }

    private static void logWindow(Window window,int eventID) {
        if (window instanceof JFrame) {
            Utils.logToConsole("detected frame entitled: " + ((JFrame) window).getTitle() + "; eventID=" + eventID);
        } else if (window instanceof JDialog) {
            Utils.logToConsole("detected dialog entitled: " + ((JDialog) window).getTitle() + "; eventID=" + eventID);
        }
    }

    static void setLoginFrame(JFrame window) {
        _LoginFrame = window;
    }

    static void setMainWindow(JFrame window) {
        Utils.logToConsole("Found TWS main window");
        _MainWindow = window;
    }

}
